<?php

/**
 * @file
 * Administrative forms for autolink_ module.
 */

/**
 * Enables a plugin via a menu callback.
 */
function autolink_plugin_enable($format, $plugin) {
  $config = autolink_config($format)
    ->enablePlugin($plugin)
    ->save();
  drupal_set_message(t('The %plugin plugin has been enabled.', array(
    '%plugin' => AutolinkConfig::$plugin_info[$plugin]['label'])));
  drupal_goto('admin/settings/filters/' . $format . '/configure');
}

/**
 * Disables a plugin via a menu callback.
 */
function autolink_plugin_disable($format, $plugin) {
  $config = autolink_config($format)
    ->disablePlugin($plugin)
    ->save();
  drupal_set_message(t('The %plugin plugin has been disabled.', array(
    '%plugin' => AutolinkConfig::$plugin_info[$plugin]['label'])));
  drupal_goto('admin/settings/filters/' . $format . '/configure');
}



/**
 * Plugin settings form.
 */
function autolink_form_plugin(&$form_state, $format, $plugin_name) {
  // Load include files that were identified in hook_autolink_file_info().
  autolink_load_includes();

  $plugin = autolink_plugin()
    ->init($plugin_name, $format);
  drupal_set_title(t('Configure plugin: %plugin', array('%plugin' => $plugin->label)));

  $form = array();
  $form['format'] = array('#type' => 'value', '#value' => $format);
  $form['plugin'] = array('#type' => 'value', '#value' => $plugin->name);
  $form['description'] = array(
    '#type' => 'item',
    '#description' => isset($plugin->description) ? $plugin->description : '',
  );

  // Retrieve the plugin defined settings form.
  $callback = $plugin->getCallback('settings');
  if (function_exists($callback)) {
    $callback($form, $plugin->settings, $plugin_name, $format);
  }

  // If the plugin does not define a form then display a message saying such.
  $children = element_children($form);
  if (empty($children)) {
    $form['configuration']['empty'] = array(
      '#value' => t('This plugin has no configuration options.'),
    );
  }
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save configuration'));
  $form['buttons']['reset'] = array(
    '#type' => 'submit',
    '#value' => t('Reset to defaults'),
    '#weight' => 10,
  );
  return $form;
}

/**
 * Validation handler for plugin configuration form.
 */
function autolink_form_plugin_validate($form, &$form_state) {
  $plugin = autolink_plugin()->init($form_state['values']['plugin'], $form_state['values']['format']);
  $callback = $plugin->getCallback('validate');
  if (function_exists($callback)) {
    $callback($form, $form_state, $form_state['values']['plugin'], $form_state['values']['format']);
  }
}

/**
 * Submit handler for the plugin configuration form.
 */
function autolink_form_plugin_submit($form, &$form_state) {
  $plugin = autolink_plugin()->init($form_state['values']['plugin'], $form_state['values']['format']);
  $op = isset($form_state['values']['op']) ? $form_state['values']['op'] : '';
  // Exclude unnecessary elements.
  unset($form_state['values']['submit'], $form_state['values']['reset'], $form_state['values']['form_id'], $form_state['values']['op'], $form_state['values']['form_token'], $form_state['values']['form_build_id']);

  if ($op == t('Reset to defaults')) {
    $plugin->reset();
  }
  else {
    foreach ($form_state['values'] as $key => $value) {
      if (is_array($value)) {
        $value = array_filter($value);
      }
      $plugin->settings[$key] = $value;
    }
  }
  $plugin->save();

  if ($op == t('Reset to defaults')) {
    drupal_set_message(t('The configuration options have been reset to their default values.'));
  }
  else {
    drupal_set_message(t('The configuration options have been saved.'));
  }
  drupal_goto('admin/settings/filters/' . $form_state['values']['format'] . '/configure');
}

/**
 * Defines the Autolink configuration interface for managing settings.
 */
interface AutolinkConfiguration {
  function save();
  function settingEnabled($setting);
}

/**
 * Defines the plugin configuration interface.
 */
interface AutolinkPluginActions {
  function enablePlugin($plugin);
  function disablePlugin($plugin);
  function pluginIsEnabled($plugin);
  function enabledPlugins();
}

/**
 * Defines the configuration class for handling Autolink administration.
 */
class AutolinkConfig implements AutolinkConfiguration, AutolinkPluginActions {

  /**
   * An array of global configuration settings, if any.
   *
   * @var array
   */
  public $settings = array();

  /**
   * An array of plugin names as keys and boolean values indicating whether
   * each individual plugin is enabled. Use array_filter() to get enabled plugins.
   *
   * @var array
   */
  public $plugins = array();

  /**
   * An array of currently configured global plugin weights.
   *
   * @var array
   */
  public $weights = array();

  /**
   * A static array of plugin information defined in hook_autolink_plugin_info().
   *
   * @var array
   */
  public static $plugin_info = array();

  /**
   * The current input format's identifier as defined by filter module.
   *
   * @var integer
   */
  public $format = -1;

  /**
   * Standard constructor method. Separates the single stored settings variable
   * into separate settings upon construction.
   */
  public function __construct($format) {
    if (empty(self::$plugin_info)) {
      self::$plugin_info = autolink_get_info('plugin');
    }
    $this->format = $format;
    $settings = variable_get('autolink_settings_' . $format, array());
    $this->settings = isset($settings['global']) ? $settings['global'] : array();
    $this->plugins = isset($settings['plugins']) ? $settings['plugins'] : array();
    $this->weights = isset($settings['weights']) ? $settings['weights'] : array();
  }

  /**
   * Returns global configuration settings when properties are undefined.
   */
  public function __get($property) {
    return isset($this->settings[$property]) ? $this->settings[$property] : FALSE;
  }

  /**
   * Returns settings to a single array and saves to a variable.
   */
  public function save() {
    $settings['global'] = $this->settings;
    $settings['plugins'] = $this->plugins;
    $settings['weights'] = $this->weights;
    variable_set('autolink_settings_' . $this->format, $settings);
    return $this;
  }

  /**
   * Enables a plugin.
   */
  public function enablePlugin($plugin) {
    $this->plugins[$plugin] = TRUE;
    return $this;
  }

  /**
   * Disables a plugin.
   */
  public function disablePlugin($plugin) {
    $this->plugins[$plugin] = FALSE;
    return $this;
  }

  /**
   * Returns a boolean value indicating whether a plugin is enabled.
   */
  public function pluginIsEnabled($plugin) {
    return !empty($this->plugins[$plugin]) && isset(self::$plugin_info[$plugin]);
  }

  /**
   * Returns all enabled plugins, ensuring that plugin info is available as well.
   */
  public function enabledPlugins() {
    $plugins = array();
    foreach (self::$plugin_info as $plugin => $info) {
      if ($this->pluginIsEnabled($plugin)) {
        $plugins[$plugin] = $plugin;
      }
    }
    return $plugins;
  }

  /**
   * Returns whether a setting is enabled.
   */
  public function settingEnabled($setting) {
    return !empty($this->settings[$setting]);
  }

}
